import 'hyperkyc_flutter_constants.dart';

/// HyperKycFlow Enum is used to define workflow
enum HyperKycFlow {
  /// Document Capture Flow
  document,

  /// Face Capture Flow
  face,
}

/// HyperKycConfig is used to launch HyperKyc
/// Create an instance of HyperKycConfig and call launch with it to start the workflow
class HyperKycConfig {
  /// AppId to launch HyperKYC : Get this from Hyperverge team
  String? appId;

  /// AppKey to launch HyperKYC : Get this from Hyperverge team
  String? appKey;

  /// AccessToken to launch HyperKYC : Please contact Hyperverge team to understand how to generate HV accessToken
  String? accessToken;

  /// TransactionId to launch HyperKYC : Please contact Hyperverge team to understand more on sessions and transactionId
  String? transactionId;

  /// Workflow : The HyperKyc SDK allows its clients to pass in a custom ordered list of HyperKycFlow types to configure the order in which capture screens and other operations happen within the SDK.
  List<Map>? workFlow;

  /// DefaultCountryId : Country will be selected by default, country-picker screen will not be shown
  String? defaultCountryId;

  /// Create HyperKycConfig instance from accessToken
  HyperKycConfig.fromAccessToken({
    required this.accessToken,
    required this.transactionId,
    required this.workFlow,
    this.defaultCountryId,
  });

  /// Create HyperKycConfig instance from appId, appKey
  HyperKycConfig.fromAppIdAppKey({
    required this.appId,
    required this.appKey,
    required this.transactionId,
    required this.workFlow,
    this.defaultCountryId,
  });

  /// Create HyperKycConfig using HyperKycConfigBuilder instance
  HyperKycConfig.fromBuilder(
      {required HyperKycConfigBuilder hyperKycConfigBuilder}) {
    if (hyperKycConfigBuilder._accessToken == null) {
      assert(
        hyperKycConfigBuilder._appId != null &&
            hyperKycConfigBuilder._appId!.isNotEmpty,
        'AppId cannot be null / empty, Please use setAppID(appId) in HyperKycConfigBuilder',
      );
      assert(
        hyperKycConfigBuilder._appKey != null &&
            hyperKycConfigBuilder._appKey!.isNotEmpty,
        'AppKey cannot be null / empty, Please use setAppKey(appKey) in HyperKycConfigBuilder',
      );
      appId = hyperKycConfigBuilder._appId!;
      appKey = hyperKycConfigBuilder._appKey!;
    } else {
      assert(
        hyperKycConfigBuilder._accessToken != null &&
            hyperKycConfigBuilder._accessToken!.isNotEmpty,
        'AccessToken cannot be null / empty, Please use setAccessToken(accessToken) in HyperKycConfigBuilder',
      );
      accessToken = hyperKycConfigBuilder._accessToken!;
    }
    assert(
      hyperKycConfigBuilder._transactionId != null &&
          hyperKycConfigBuilder._transactionId!.isNotEmpty,
      'TransactionId cannot be null / empty, Please use setTransactionId(transactionId) in HyperKycConfigBuilder',
    );
    assert(
      hyperKycConfigBuilder._workFlow != null,
      'WorkFlow cannot be null, Please use setWorkFlow(workFlow) in HyperKycConfigBuilder',
    );
    transactionId = hyperKycConfigBuilder._transactionId!;
    workFlow = hyperKycConfigBuilder._workFlow!;
    defaultCountryId = hyperKycConfigBuilder._defaultCountryId;
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycConfig{appId: $appId, appKey: $appKey, accessToken: $accessToken, transactionId: $transactionId, workFlow: $workFlow, defaultCountryId: $defaultCountryId}';
  }

  /// Convert HyperKycConfig object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.appId: this.appId,
      HyperKycConstants.appKey: this.appKey,
      HyperKycConstants.accessToken: this.accessToken,
      HyperKycConstants.transactionId: this.transactionId,
      HyperKycConstants.workFlow: this.workFlow,
      HyperKycConstants.defaultCountryId: this.defaultCountryId,
    };
  }
}

/// Builder class to build HyperKycConfig
class HyperKycConfigBuilder {
  /// AppID to launch HyperKYC : Get this from Hyperverge team
  String? _appId;

  /// AppKey to launch HyperKYC : Get this from Hyperverge team
  String? _appKey;

  /// AccessToken to launch HyperKYC : Please contact Hyperverge team to understand how to generate HV accessToken
  String? _accessToken;

  /// TransactionId to launch HyperKYC : Please contact Hyperverge team to understand more on sessions and transactionId
  String? _transactionId;

  /// Workflow : The HyperKyc SDK allows its clients to pass in a custom ordered list of HyperKycFlow types to configure the order in which capture screens and other operations happen within the SDK.
  List<Map>? _workFlow;

  /// DefaultCountryId : Country will be selected by default, country-picker screen will not be shown
  String? _defaultCountryId;

  /// Default Constructor to create HyperKycConfigBuilder instance
  HyperKycConfigBuilder();

  /// static HyperKycConfigBuilder instance
  static final HyperKycConfigBuilder hyperKycConfigBuilder =
      HyperKycConfigBuilder();

  /// Use this setter to set HyperKycConfig appId
  HyperKycConfigBuilder setAppId({required String appId}) {
    hyperKycConfigBuilder._appId = appId;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig appKey
  HyperKycConfigBuilder setAppKey({required String appKey}) {
    hyperKycConfigBuilder._appKey = appKey;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig accessToken
  HyperKycConfigBuilder setAccessToken({required String accessToken}) {
    hyperKycConfigBuilder._accessToken = accessToken;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig transactionId
  HyperKycConfigBuilder setTransactionId({required String transactionId}) {
    hyperKycConfigBuilder._transactionId = transactionId;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig workFlow
  HyperKycConfigBuilder setWorkFlow({required List<Map> workFlow}) {
    hyperKycConfigBuilder._workFlow = workFlow;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig defaultCountryId
  HyperKycConfigBuilder setDefaultCountryId(
      {required String defaultCountryId}) {
    hyperKycConfigBuilder._defaultCountryId = defaultCountryId;
    return hyperKycConfigBuilder;
  }

  /// Use this build function to create HyperKycConfig instance from builder properties
  HyperKycConfig build() {
    return HyperKycConfig.fromBuilder(
        hyperKycConfigBuilder: hyperKycConfigBuilder);
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycConfigBuilder{_appId: $_appId, _appKey: $_appKey, _accessToken: $_accessToken, _transactionId: $_transactionId, _workFlow: $_workFlow, _defaultCountryId: $_defaultCountryId}';
  }
}
