import 'dart:convert';

import 'package:hyperkyc_flutter/hyperkyc_api_base_response.dart';

import 'hyperkyc_flutter_constants.dart';

/// DocResult contains List of DocData
class DocResult {
  /// tag for which this docResult belongs to
  final String? tag;

  /// documentId for which this docResult belongs to
  final String? documentId;

  /// List of document capture and OCR results based on no. of sides of the selected document
  final List<DocData?>? docDataList;

  /// Constructor
  DocResult({
    this.tag,
    this.documentId,
    this.docDataList,
  });

  /// Convert Map object to DocResultObject
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.tag: this.tag,
      HyperKycConstants.documentId: this.documentId,
      HyperKycConstants.docDataList: this.docDataList,
    };
  }

  /// Convert DocResult object to Map
  factory DocResult.fromMap(Map<dynamic, dynamic> map) {
    List<DocData> docDataList = List.empty(growable: true);
    List docDataListFromMap = map[HyperKycConstants.docDataList] ?? [];
    for (var docDataMap in docDataListFromMap) {
      DocData docData = DocData.fromMap(docDataMap ?? {});
      docDataList.add(docData);
    }
    return DocResult(
      tag: map[HyperKycConstants.tag] as String?,
      documentId: map[HyperKycConstants.documentId] as String?,
      docDataList: docDataList,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'DocResult{tag: $tag, documentId: $documentId, docDataList: $docDataList}';
  }
}

/// DocData contains document capture and OCR results based on no. of sides of the selected document
class DocData {
  /// Document side - front / back
  final String? side;

  /// File path to the captured document image
  final String? docImagePath;

  /// Action - pass / manualReview / fail
  final String? action;

  /// Response headers (json string) from OCR API
  final String? responseHeaders;

  /// Response body from the OCR API
  final DocCaptureApiDetail? responseResult;

  /// Constructor
  DocData({
    this.side,
    this.docImagePath,
    this.action,
    this.responseHeaders,
    this.responseResult,
  });

  /// Convert Map to DocData object
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.side: this.side,
      HyperKycConstants.docImagePath: this.docImagePath,
      HyperKycConstants.action: this.action,
      HyperKycConstants.responseHeaders: this.responseHeaders,
      HyperKycConstants.responseResult: this.responseResult,
    };
  }

  /// Convert DocData object to Map
  factory DocData.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return DocData();
    }
    return DocData(
      side: map[HyperKycConstants.side] as String?,
      docImagePath: map[HyperKycConstants.docImagePath] as String?,
      action: map[HyperKycConstants.action] as String?,
      responseHeaders: map[HyperKycConstants.responseHeaders] as String?,
      responseResult: DocCaptureApiDetail.fromMap(
          jsonDecode(map[HyperKycConstants.responseResult]) ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'DocData{side: $side, docImagePath: $docImagePath, action: $action, responseHeaders: $responseHeaders, responseResult: $responseResult}';
  }
}

/// DocCaptureApiDetail is response body from the Hyperverge OCR API call
class DocCaptureApiDetail {
  /// Metadata contains data like TransactionId, RequestId, Attempts count, etc
  final Metadata? metadata;

  /// Status received after Hyperverge API call
  final String? status;

  /// StatusCode received after Hyperverge API call
  final String? statusCode;

  /// DocResultObject
  final DocResultObject? result;

  /// Constructor
  DocCaptureApiDetail({
    this.metadata,
    this.status,
    this.statusCode,
    this.result,
  });

  /// Convert DocCaptureApiDetail object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.metadata: this.metadata,
      HyperKycConstants.status: this.status,
      HyperKycConstants.statusCode: this.statusCode,
      HyperKycConstants.result: this.result,
    };
  }

  /// Convert Map to DocCaptureApiDetail object
  factory DocCaptureApiDetail.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return DocCaptureApiDetail();
    }
    return DocCaptureApiDetail(
      metadata: Metadata.fromMap(map[HyperKycConstants.metadata] ?? {}),
      status: map[HyperKycConstants.status] as String?,
      statusCode: (map[HyperKycConstants.statusCode] ?? '').toString(),
      result: DocResultObject.fromMap(map[HyperKycConstants.result] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'DocCaptureApiDetail{metadata: $metadata, status: $status, statusCode: $statusCode, result: $result}';
  }
}

/// DocResultObject
class DocResultObject {
  /// List of details present in OCR result
  final List<OCRApiStructure?>? details;

  /// Error
  final String? error;

  /// Summary present in OCR result
  final Summary? summary;

  /// Constructor
  DocResultObject({
    this.details,
    this.error,
    this.summary,
  });

  /// Convert DocResultObject object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.details: this.details,
      HyperKycConstants.error: this.error,
      HyperKycConstants.summary: this.summary,
    };
  }

  /// Convert Map to DocResultObject object
  factory DocResultObject.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return DocResultObject();
    }
    List<OCRApiStructure> ocrApiStructureList = List.empty(growable: true);
    List details = map[HyperKycConstants.details] ?? [];
    if (details.isNotEmpty) {
      for (var detail in details) {
        OCRApiStructure ocrApiStructureDetail =
            OCRApiStructure.fromMap(detail ?? {});
        ocrApiStructureList.add(ocrApiStructureDetail);
      }
    }
    return DocResultObject(
      details: ocrApiStructureList,
      error: map[HyperKycConstants.error] as String?,
      summary: Summary.fromMap(map[HyperKycConstants.summary] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'DocResultObject{details: $details, error: $error, summary: $summary}';
  }
}

/// OCRApiStructure is the blueprint of response result details received from Hyperverge API call
class OCRApiStructure {
  /// idType : PAN / Aadhaar / Social Security Card / Passport etc
  final String? idType;

  /// FieldsExtracted
  final FieldsExtracted? fieldsExtracted;

  /// DocQualityChecks
  final DocQualityChecks? qualityChecks;

  /// RuleChecks
  final RuleChecks? ruleChecks;

  /// Constructor
  OCRApiStructure({
    this.idType,
    this.fieldsExtracted,
    this.qualityChecks,
    this.ruleChecks,
  });

  /// Convert OCRApiStructure object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.idType: this.idType,
      HyperKycConstants.fieldsExtracted: this.fieldsExtracted,
      HyperKycConstants.qualityChecks: this.qualityChecks,
      HyperKycConstants.ruleChecks: this.ruleChecks,
    };
  }

  /// Convert Map to OCRApiStructure object
  factory OCRApiStructure.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return OCRApiStructure();
    }
    return OCRApiStructure(
      idType: map[HyperKycConstants.idType] as String?,
      fieldsExtracted:
          FieldsExtracted.fromMap(map[HyperKycConstants.fieldsExtracted] ?? {}),
      qualityChecks:
          DocQualityChecks.fromMap(map[HyperKycConstants.qualityChecks] ?? {}),
      ruleChecks: RuleChecks.fromMap(map[HyperKycConstants.ruleChecks] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'OCRApiStructure{idType: $idType, fieldsExtracted: $fieldsExtracted, qualityChecks: $qualityChecks, ruleChecks: $ruleChecks}';
  }
}

/// FieldsExtracted from the response received from Hyperverge API call
class FieldsExtracted {
  /// idNumber extracted from the document
  final Fieldvalue? idNumber;

  /// FirstName extracted from the document
  final Fieldvalue? firstName;

  /// MiddleName extracted from the document
  final Fieldvalue? middleName;

  /// LastName extracted from the document
  final Fieldvalue? lastName;

  /// FullName extracted from the document
  final Fieldvalue? fullName;

  /// Date of birth extracted from the document
  final Fieldvalue? dateOfBirth;

  /// Date of issue extracted from the document
  final Fieldvalue? dateOfIssue;

  /// Date of expiry extracted from the document
  final Fieldvalue? dateOfExpiry;

  /// Country Code extracted from the document
  final Fieldvalue? countryCode;

  /// Type extracted from the document
  final Fieldvalue? type;

  /// Gender extracted from the document
  final Fieldvalue? gender;

  /// Place of birth extracted from the document
  final Fieldvalue? placeOfBirth;

  /// Place of issue extracted from the document
  final Fieldvalue? placeOfIssue;

  /// Year of birth extracted from the document
  final Fieldvalue? yearOfBirth;

  /// Age extracted from the document
  final Fieldvalue? age;

  /// FatherName extracted from the document
  final Fieldvalue? fatherName;

  /// MotherName extracted from the document
  final Fieldvalue? motherName;

  /// HusbandName extracted from the document
  final Fieldvalue? husbandName;

  /// SpouseName extracted from the document
  final Fieldvalue? spouseName;

  /// Nationality extracted from the document
  final Fieldvalue? nationality;

  /// MRZ String extracted from the document
  final Fieldvalue? mrzString;

  /// Hometown extracted from the document
  final Fieldvalue? hometown;

  /// Address extracted from the document
  final Address? address;

  /// Constructor
  FieldsExtracted({
    this.idNumber,
    this.firstName,
    this.middleName,
    this.lastName,
    this.fullName,
    this.dateOfBirth,
    this.dateOfIssue,
    this.dateOfExpiry,
    this.countryCode,
    this.type,
    this.gender,
    this.placeOfBirth,
    this.placeOfIssue,
    this.yearOfBirth,
    this.age,
    this.fatherName,
    this.motherName,
    this.husbandName,
    this.spouseName,
    this.nationality,
    this.mrzString,
    this.hometown,
    this.address,
  });

  /// Convert FieldsExtracted object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.idNumber: this.idNumber,
      HyperKycConstants.firstName: this.firstName,
      HyperKycConstants.middleName: this.middleName,
      HyperKycConstants.lastName: this.lastName,
      HyperKycConstants.fullName: this.fullName,
      HyperKycConstants.dateOfBirth: this.dateOfBirth,
      HyperKycConstants.dateOfIssue: this.dateOfIssue,
      HyperKycConstants.dateOfExpiry: this.dateOfExpiry,
      HyperKycConstants.countryCode: this.countryCode,
      HyperKycConstants.type: this.type,
      HyperKycConstants.gender: this.gender,
      HyperKycConstants.placeOfBirth: this.placeOfBirth,
      HyperKycConstants.placeOfIssue: this.placeOfIssue,
      HyperKycConstants.yearOfBirth: this.yearOfBirth,
      HyperKycConstants.age: this.age,
      HyperKycConstants.fatherName: this.fatherName,
      HyperKycConstants.motherName: this.motherName,
      HyperKycConstants.husbandName: this.husbandName,
      HyperKycConstants.spouseName: this.spouseName,
      HyperKycConstants.nationality: this.nationality,
      HyperKycConstants.mrzString: this.mrzString,
      HyperKycConstants.hometown: this.hometown,
      HyperKycConstants.address: this.address,
    };
  }

  /// Convert Map to FieldsExtracted object
  factory FieldsExtracted.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return FieldsExtracted();
    }
    return FieldsExtracted(
      idNumber: Fieldvalue.fromMap(map[HyperKycConstants.idNumber] ?? {}),
      firstName: Fieldvalue.fromMap(map[HyperKycConstants.firstName] ?? {}),
      middleName: Fieldvalue.fromMap(map[HyperKycConstants.middleName] ?? {}),
      lastName: Fieldvalue.fromMap(map[HyperKycConstants.lastName] ?? {}),
      fullName: Fieldvalue.fromMap(map[HyperKycConstants.fullName] ?? {}),
      dateOfBirth: Fieldvalue.fromMap(map[HyperKycConstants.dateOfBirth] ?? {}),
      dateOfIssue: Fieldvalue.fromMap(map[HyperKycConstants.dateOfIssue] ?? {}),
      dateOfExpiry:
          Fieldvalue.fromMap(map[HyperKycConstants.dateOfExpiry] ?? {}),
      countryCode: Fieldvalue.fromMap(map[HyperKycConstants.countryCode] ?? {}),
      type: Fieldvalue.fromMap(map[HyperKycConstants.type] ?? {}),
      gender: Fieldvalue.fromMap(map[HyperKycConstants.gender] ?? {}),
      placeOfBirth:
          Fieldvalue.fromMap(map[HyperKycConstants.placeOfBirth] ?? {}),
      placeOfIssue:
          Fieldvalue.fromMap(map[HyperKycConstants.placeOfIssue] ?? {}),
      yearOfBirth: Fieldvalue.fromMap(map[HyperKycConstants.yearOfBirth] ?? {}),
      age: Fieldvalue.fromMap(map[HyperKycConstants.age] ?? {}),
      fatherName: Fieldvalue.fromMap(map[HyperKycConstants.fatherName] ?? {}),
      motherName: Fieldvalue.fromMap(map[HyperKycConstants.motherName] ?? {}),
      husbandName: Fieldvalue.fromMap(map[HyperKycConstants.husbandName] ?? {}),
      spouseName: Fieldvalue.fromMap(map[HyperKycConstants.spouseName] ?? {}),
      nationality: Fieldvalue.fromMap(map[HyperKycConstants.nationality] ?? {}),
      mrzString: Fieldvalue.fromMap(map[HyperKycConstants.mrzString] ?? {}),
      hometown: Fieldvalue.fromMap(map[HyperKycConstants.hometown] ?? {}),
      address: Address.fromMap(map[HyperKycConstants.address] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FieldsExtracted{idNumber: $idNumber, firstName: $firstName, middleName: $middleName, lastName: $lastName, fullName: $fullName, dateOfBirth: $dateOfBirth, dateOfIssue: $dateOfIssue, dateOfExpiry: $dateOfExpiry, countryCode: $countryCode, type: $type, gender: $gender, placeOfBirth: $placeOfBirth, placeOfIssue: $placeOfIssue, yearOfBirth: $yearOfBirth, age: $age, fatherName: $fatherName, motherName: $motherName, husbandName: $husbandName, spouseName: $spouseName, nationality: $nationality, mrzString: $mrzString, hometown: $hometown, address: $address}';
  }
}

/// Address holds the properties of Address received from Hyperverge API call
class Address {
  /// Value
  final String? value;

  /// Confidence
  final String? confidence;

  /// Confidence score
  final String? score;

  /// House Number
  final String? houseNumber;

  /// Additional info
  final String? additionalInfo;

  /// Province
  final String? province;

  /// District
  final String? district;

  /// Street
  final String? street;

  /// Zipcode
  final String? zipCode;

  /// Constructor
  Address({
    this.value,
    this.confidence,
    this.score,
    this.houseNumber,
    this.additionalInfo,
    this.province,
    this.district,
    this.street,
    this.zipCode,
  });

  /// Convert Address object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.value: this.value,
      HyperKycConstants.confidence: this.confidence,
      HyperKycConstants.score: this.score,
      HyperKycConstants.houseNumber: this.houseNumber,
      HyperKycConstants.additionalInfo: this.additionalInfo,
      HyperKycConstants.province: this.province,
      HyperKycConstants.district: this.district,
      HyperKycConstants.street: this.street,
      HyperKycConstants.zipCode: this.zipCode,
    };
  }

  /// Convert Map to Address object
  factory Address.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return Address();
    }
    return Address(
      value: map[HyperKycConstants.value] as String?,
      confidence: map[HyperKycConstants.confidence] as String?,
      score: map[HyperKycConstants.score] as String?,
      houseNumber: map[HyperKycConstants.houseNumber] as String?,
      additionalInfo: map[HyperKycConstants.additionalInfo] as String?,
      province: map[HyperKycConstants.province] as String?,
      district: map[HyperKycConstants.district] as String?,
      street: map[HyperKycConstants.street] as String?,
      zipCode: map[HyperKycConstants.zipCode] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'Address{value: $value, confidence: $confidence, score: $score, houseNumber: $houseNumber, additionalInfo: $additionalInfo, province: $province, district: $district, street: $street, zipCode: $zipCode}';
  }
}

/// DocQualityChecks
class DocQualityChecks {
  /// Blur check
  final Fieldvalue? blur;

  ///  Glare check
  final Fieldvalue? glare;

  ///  blackAndWhite check
  final Fieldvalue? blackAndWhite;

  ///  capturedFromScreen check
  final Fieldvalue? capturedFromScreen;

  ///  whiteBackground check
  final Fieldvalue? whiteBackground;

  ///  cutId check
  final Fieldvalue? cutId;

  /// Constructor
  DocQualityChecks({
    this.blur,
    this.glare,
    this.blackAndWhite,
    this.capturedFromScreen,
    this.whiteBackground,
    this.cutId,
  });

  /// Convert DocQualityChecks object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.blur: this.blur,
      HyperKycConstants.glare: this.glare,
      HyperKycConstants.blackAndWhite: this.blackAndWhite,
      HyperKycConstants.capturedFromScreen: this.capturedFromScreen,
      HyperKycConstants.whiteBackground: this.whiteBackground,
      HyperKycConstants.cutId: this.cutId,
    };
  }

  /// Convert Map to DocQualityChecks object
  factory DocQualityChecks.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return DocQualityChecks();
    }
    return DocQualityChecks(
      blur: Fieldvalue.fromMap(map[HyperKycConstants.blur] ?? {}),
      glare: Fieldvalue.fromMap(map[HyperKycConstants.glare] ?? {}),
      blackAndWhite:
          Fieldvalue.fromMap(map[HyperKycConstants.blackAndWhite] ?? {}),
      capturedFromScreen:
          Fieldvalue.fromMap(map[HyperKycConstants.capturedFromScreen] ?? {}),
      whiteBackground:
          Fieldvalue.fromMap(map[HyperKycConstants.whiteBackground] ?? {}),
      cutId: Fieldvalue.fromMap(map[HyperKycConstants.cutId] ?? {}),
    );
  }

  /// Overriden toString() method@override
  String toString() {
    return 'DocQualityChecks{blur: $blur, glare: $glare, blackAndWhite: $blackAndWhite, capturedFromScreen: $capturedFromScreen, whiteBackground: $whiteBackground, cutId: $cutId}';
  }
}

/// RuleChecks
class RuleChecks {
  /// id number invalid check
  final Fieldvalue? idNumberInvalid;

  /// Constructor
  RuleChecks({
    this.idNumberInvalid,
  });

  ///  Convert RuleChecks object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.idNumberInvalid: this.idNumberInvalid,
    };
  }

  /// Convert Map to RuleChecks object
  factory RuleChecks.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return RuleChecks();
    }
    return RuleChecks(
      idNumberInvalid:
          Fieldvalue.fromMap(map[HyperKycConstants.idNumberInvalid] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'RuleChecks{idNumberInvalid: $idNumberInvalid}';
  }
}
