import 'dart:convert';

import 'package:hyperkyc_flutter/hyperkyc_api_base_response.dart';

import 'hyperkyc_flutter_constants.dart';

/// FaceData contains face capture and liveness result
class FaceData {
  /// File path to the cropped face image
  final String? croppedFaceImagePath;

  /// File path to the full face image
  final String? fullFaceImagePath;

  /// File path to the face capture video if available
  final String? videoPath;

  /// Action - pass / manualReview / fail
  final String? action;

  /// Response headers (json string) from liveness API
  final String? responseHeaders;

  /// Response body from the liveness API
  final FaceCaptureApiDetail? responseResult;

  /// Constructor
  FaceData({
    this.croppedFaceImagePath,
    this.fullFaceImagePath,
    this.videoPath,
    this.action,
    this.responseHeaders,
    this.responseResult,
  });

  /// Convert FaceData object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.croppedFaceImagePath: this.croppedFaceImagePath,
      HyperKycConstants.fullFaceImagePath: this.fullFaceImagePath,
      HyperKycConstants.videoPath: this.videoPath,
      HyperKycConstants.action: this.action,
      HyperKycConstants.responseHeaders: this.responseHeaders,
      HyperKycConstants.responseResult: this.responseResult,
    };
  }

  /// Convert Map to FaceData object
  factory FaceData.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return FaceData();
    }
    return FaceData(
      croppedFaceImagePath:
          map[HyperKycConstants.croppedFaceImagePath] as String?,
      fullFaceImagePath: map[HyperKycConstants.fullFaceImagePath] as String?,
      videoPath: map[HyperKycConstants.videoPath] as String?,
      action: map[HyperKycConstants.action] as String?,
      responseHeaders: map[HyperKycConstants.responseHeaders] as String?,
      responseResult: FaceCaptureApiDetail.fromMap(
          jsonDecode(map[HyperKycConstants.responseResult]) ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceData{croppedFaceImagePath: $croppedFaceImagePath, fullFaceImagePath: $fullFaceImagePath, videoPath: $videoPath, action: $action, responseHeaders: $responseHeaders, responseResult: $responseResult}';
  }
}

/// FaceCaptureApiDetail is response body from the Hyperverge liveness API call
class FaceCaptureApiDetail {
  /// Metadata contains data like TransactionId, RequestId, Attempts count, etc
  final Metadata? metadata;

  /// Status received after Hyperverge API call
  final String? status;

  /// StatusCode received after Hyperverge API call
  final String? statusCode;

  /// FaceResultObject
  final FaceResultObject? result;

  /// Constructor
  FaceCaptureApiDetail({
    this.metadata,
    this.status,
    this.statusCode,
    this.result,
  });

  /// Convert FaceCaptureApiDetail object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.metadata: this.metadata,
      HyperKycConstants.status: this.status,
      HyperKycConstants.statusCode: this.statusCode,
      HyperKycConstants.result: this.result,
    };
  }

  /// Convert Map to FaceCaptureApiDetail object
  factory FaceCaptureApiDetail.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return FaceCaptureApiDetail();
    }
    return FaceCaptureApiDetail(
      metadata: Metadata.fromMap(map[HyperKycConstants.metadata] ?? {}),
      status: map[HyperKycConstants.status] as String?,
      statusCode: (map[HyperKycConstants.statusCode] ?? '').toString(),
      result: FaceResultObject.fromMap(map[HyperKycConstants.result] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceCaptureApiDetail{metadata: $metadata, status: $status, statusCode: $statusCode, result: $result}';
  }
}

/// FaceResultObject
class FaceResultObject {
  /// Details present in liveness result
  final LivenessApiStructure? details;

  /// Error
  final String? error;

  /// Summary
  final Summary? summary;

  /// Constructor
  FaceResultObject({
    this.details,
    this.error,
    this.summary,
  });

  /// Convert FaceResultObject object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.details: this.details,
      HyperKycConstants.error: this.error,
      HyperKycConstants.summary: this.summary,
    };
  }

  /// Convert Map to FaceResultObject object
  factory FaceResultObject.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return FaceResultObject();
    }
    return FaceResultObject(
      details:
          LivenessApiStructure.fromMap(map[HyperKycConstants.details] ?? {}),
      error: map[HyperKycConstants.error] as String?,
      summary: Summary.fromMap(map[HyperKycConstants.summary] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceResultObject{details: $details, error: $error, summary: $summary}';
  }
}

/// LivenessApiStructure
class LivenessApiStructure {
  /// LiveFace
  final Fieldvalue? liveFace;

  /// Live
  final String? live;

  /// Liveness score
  final String? livenessScore;

  /// to be reviewed or not
  final String? toBeReviewed;

  /// Face Quality checks
  final FaceQualityChecks? qualityChecks;

  /// Error
  final String? error;

  /// Constructor
  LivenessApiStructure({
    this.liveFace,
    this.live,
    this.livenessScore,
    this.toBeReviewed,
    this.qualityChecks,
    this.error,
  });

  /// Convert LivenessApiStructure object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.liveFace: this.liveFace,
      HyperKycConstants.live: this.live,
      HyperKycConstants.livenessScore: this.livenessScore,
      HyperKycConstants.toBeReviewed: this.toBeReviewed,
      HyperKycConstants.qualityChecks: this.qualityChecks,
      HyperKycConstants.error: this.error,
    };
  }

  /// Convert Map to LivenessApiStructure object
  factory LivenessApiStructure.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return LivenessApiStructure();
    }
    return LivenessApiStructure(
      liveFace: Fieldvalue.fromMap(map[HyperKycConstants.liveFace] ?? {}),
      live: map[HyperKycConstants.live] as String?,
      livenessScore: map[HyperKycConstants.livenessScore] as String?,
      toBeReviewed: map[HyperKycConstants.toBeReviewed] as String?,
      qualityChecks:
          FaceQualityChecks.fromMap(map[HyperKycConstants.qualityChecks] ?? {}),
      error: map[HyperKycConstants.error] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'LivenessApiStructure{liveFace: $liveFace, live: $live, livenessScore: $livenessScore, toBeReviewed: $toBeReviewed, qualityChecks: $qualityChecks, error: $error}';
  }
}

/// FaceQualityChecks
class FaceQualityChecks {
  /// Eyes Closed check
  final Fieldvalue? eyesClosed;

  /// Mask present check
  final Fieldvalue? maskPresent;

  /// Multiple faces check
  final Fieldvalue? multipleFaces;

  /// Blur check
  final Fieldvalue? blur;

  /// Constructor
  FaceQualityChecks({
    this.eyesClosed,
    this.maskPresent,
    this.multipleFaces,
    this.blur,
  });

  /// Convert FaceQualityChecks object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.eyesClosed: this.eyesClosed,
      HyperKycConstants.maskPresent: this.maskPresent,
      HyperKycConstants.multipleFaces: this.multipleFaces,
      HyperKycConstants.blur: this.blur,
    };
  }

  /// Convert Map to FaceQualityChecks object
  factory FaceQualityChecks.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return FaceQualityChecks();
    }
    return FaceQualityChecks(
      eyesClosed: Fieldvalue.fromMap(map[HyperKycConstants.eyesClosed] ?? {}),
      maskPresent: Fieldvalue.fromMap(map[HyperKycConstants.maskPresent] ?? {}),
      multipleFaces:
          Fieldvalue.fromMap(map[HyperKycConstants.multipleFaces] ?? {}),
      blur: Fieldvalue.fromMap(map[HyperKycConstants.blur] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceQualityChecks{eyesClosed: $eyesClosed, maskPresent: $maskPresent, multipleFaces: $multipleFaces, blur: $blur}';
  }
}
